@@ -1,5 +1,5 @@
 package ZMQ::Raw::Loop::Timer;
-$ZMQ::Raw::Loop::Timer::VERSION = '0.18';
+$ZMQ::Raw::Loop::Timer::VERSION = '0.19';
 use strict;
 use warnings;
 use Carp;
@@ -34,7 +34,7 @@ ZMQ::Raw::Loop::Timer - Timer class
 
 =head1 VERSION
 
-version 0.18
+version 0.19
 
 =head1 DESCRIPTION
 
@@ -123,7 +123,6 @@ sub cancel
 	if ($this->loop)
 	{
 		$this->loop->remove ($this);
-		$this->loop (undef);
 	}
 }
 
@@ -134,6 +133,12 @@ sub reset
 	my ($this) = @_;
 
 	$this->timer->reset;
+
+	if ($this->loop)
+	{
+		$this->loop->remove ($this);
+		$this->loop->add ($this);
+	}
 }
 
 
